@@ -114,6 +114,7 @@ import types
 import traceback
 from . import js_context
 from .hex_codec import hex_to_bytearray, bytearray_to_hex
+from pprint import pprint
 
 # In the IPython context get_ipython is a builtin.
 # get a reference to the IPython notebook object.
@@ -136,18 +137,24 @@ LOAD_CSS = "load_css"
 LOAD_JS = "load_js"
 LOAD_INDICATORS = [LOAD_CSS, LOAD_JS]
 
+# This slot name is used to support D3-style chaining under some circumstances
+FRAGILE_JS_REFERENCE = "_FRAGILE_JS_REFERENCE"
+
+# Reference used for method resolution
+FRAGILE_THIS = "_FRAGILE_THIS"
+
 # Message segmentation size default
 BIG_SEGMENT = 1000000
 
 @widgets.register
 class JSProxyWidget(widgets.DOMWidget):
-    """Introspective javascript proxy widget."""
+    """Introspective javascript proxy widget.""" 
     _view_name = Unicode('JSProxyView').tag(sync=True)
     _model_name = Unicode('JSProxyModel').tag(sync=True)
     _view_module = Unicode('jp_proxy_widget').tag(sync=True)
     _model_module = Unicode('jp_proxy_widget').tag(sync=True)
-    _view_module_version = Unicode('^1.0.2').tag(sync=True)
-    _model_module_version = Unicode('^1.0.2').tag(sync=True)
+    _view_module_version = Unicode('^1.0.4').tag(sync=True)
+    _model_module_version = Unicode('^1.0.4').tag(sync=True)
 
     # traitlet port to use for sending commands to javascript
     #commands = traitlets.List([], sync=True)
@@ -188,6 +195,41 @@ class JSProxyWidget(widgets.DOMWidget):
         self.last_callback_results = None
         self.results = []
         self.status = "Not yet rendered"
+        self.last_attribute = None
+        # Used for D3 style "chaining" -- reference to last object reference cached on JS side
+        self.last_fragile_reference = None
+        #self.executing_fragile = False
+        # standard initialization in javascript
+        self.js_init("""
+            // make the window accessible through the element
+            element.window = window;
+            element._FRAGILE_THIS = null;
+        """)
+
+    def execute_and_return_fragile_reference(self, for_action):
+        "This is a trick to support D3-style chaining of Python expressions that map to Javascript."
+        #if self.executing_fragile:
+        #    raise ValueError("recursing in fragile execution")
+        #self.executing_fragile = True
+        #("execute_and_return_fragile_reference(")
+        # execute the action and cache the result temporarily
+        #self.element._set(FRAGILE_JS_REFERENCE, for_action)
+        # DON'T execute it twice
+        #self(cached)
+        #reference = self.element[FRAGILE_JS_REFERENCE]
+        #self.executing_fragile = False
+        #reference = ["get", ["element"], FRAGILE_JS_REFERENCE]
+        #execute_action = ["set", ["element"], FRAGILE_JS_REFERENCE, for_action]
+        # Save action stores value used as "this" in chained method calls
+        save_action = SetMaker(
+            self.get_element(), 
+            FRAGILE_THIS,
+            MethodMaker(self.get_element(), FRAGILE_JS_REFERENCE)
+            )
+        execute_action = SetMaker(self.get_element(), FRAGILE_JS_REFERENCE, for_action)
+        self.send_commands([save_action, execute_action])
+        reference = MethodMaker(self.get_element(), FRAGILE_JS_REFERENCE)
+        return FragileReference(self, reference, for_action)
 
     def set_element(self, slot_name, value):
         """
@@ -215,6 +257,7 @@ class JSProxyWidget(widgets.DOMWidget):
         #pr ("js_init")
         #pr(js_function_body)
         other_argument_names = list(other_arguments.keys())
+        #pr ("other names", other_argument_names)
         def listiffy(v):
             "convert tuples and elements of tuples to lists"
             type_v = type(v)
@@ -225,10 +268,14 @@ class JSProxyWidget(widgets.DOMWidget):
             else:
                 return v
         def map_value(v):
-            if callable(v):
-                return self.callable(v, level=callable_level)
+            #if callable(v):
+            #    return self.callable(v, level=callable_level)
+            v = self.wrap_callables(v, callable_level=callable_level)
             return listiffy(v)
         other_argument_values = [map_value(other_arguments[name]) for name in other_argument_names]
+        #pr( "other_values", other_argument_values)
+        #other_argument_values = [other_arguments[name] for name in other_argument_names]
+        #other_argument_values = self.wrap_callables(other_argument_values)
         argument_names = list(["element"] + other_argument_names)
         argument_values = list([self.get_element()] + other_argument_values)
         function = self.function(argument_names, js_function_body)
@@ -244,6 +291,20 @@ class JSProxyWidget(widgets.DOMWidget):
             # just execute it
             action()
 
+    def wrap_callables(self, x, callable_level=3):
+        if callable(x) and not isinstance(x, CommandMakerSuperClass):
+            return self.callable(x, level=callable_level)
+        w = self.wrap_callables
+        ty = type(x)
+        if ty is list:
+            return list(w(y) for y in x)
+        if ty is tuple:
+            return tuple(w(y) for y in x)
+        if ty is dict:
+            return dict((k, w(v)) for (k,v) in x.items())
+        # default
+        return x
+
     print_on_error = True
 
     def handle_error_msg(self, att_name, old, new):
@@ -265,6 +326,11 @@ class JSProxyWidget(widgets.DOMWidget):
             INDICATOR: indicator,
             PAYLOAD: payload,
         }
+        # XXXX debug
+        self._last_payload = payload
+        #pr("sending")
+        #pprint(package)
+        debug_check_commands(package)
         self.send(package)
 
     # slot for last message data debugging
@@ -302,9 +368,11 @@ class JSProxyWidget(widgets.DOMWidget):
         return assembly
 
     def handle_custom_message(self, widget, data, *etcetera):
+        #pr("handle custom message")
+        #pprint(data)
         try:
             self._last_message_data = data
-            indicator = data[INDICATOR];
+            indicator = data[INDICATOR]
             payload = data[PAYLOAD]
             if indicator == RESULTS:
                 self.results = payload
@@ -341,7 +409,8 @@ class JSProxyWidget(widgets.DOMWidget):
 
     def __call__(self, command):
         "Add a command to the buffered commands. Convenience."
-        self.buffered_commands.append(command)
+        #self.buffered_commands.append(command)
+        self.send_command(command)
         if self.auto_flush:
             self.flush()
         return command
@@ -365,6 +434,9 @@ class JSProxyWidget(widgets.DOMWidget):
         value is a reference to the element by name.
         """
         elt = self.get_element()
+        if callable(reference):
+            # convert to javascript callback
+            reference = self.callable(reference)  # XXXX should generalize this to allow callablse of lists etc
         save_command = elt._set(name, reference)
         # buffer the save operation
         self(save_command)
@@ -533,6 +605,8 @@ class JSProxyWidget(widgets.DOMWidget):
 
     def handle_callback_results(self, new):
         "Callback for when the JS View sends an event notification."
+        #pr ("HANDLE CALLBACK RESULTS")
+        #pprint(new)
         self.last_callback_results = new
         if self.verbose:
             print ("got callback results", new)
@@ -553,15 +627,18 @@ class JSProxyWidget(widgets.DOMWidget):
         "Send a single command to the JS View."
         return self.send_commands([command], results_callback, level)
 
-    def send_commands(self, commands_iter, results_callback=None, level=1, segmented=None):
+    def send_commands(self, commands_iter, results_callback=None, level=1, segmented=None, check=False):
         """Send several commands fo the JS View.
         If segmented is a positive integer then the commands payload will be pre-encoded
         as a json string and sent in segments of that length
         """
         count = self.counter
         self.counter = count + 1
+        commands_iter = list(commands_iter)
         qcommands = list(map(quoteIfNeeded, commands_iter))
-        commands = validate_commands(qcommands)
+        commands = self.validate_commands(qcommands)
+        if check:
+            debug_check_commands(commands)
         if self.rendered:
             # also send any commands awaiting the render event.
             if self.commands_awaiting_render:
@@ -654,6 +731,7 @@ class JSProxyWidget(widgets.DOMWidget):
                     count += 1
                 else:
                     break
+            #p("XXXXX callback args", py_arguments)
             function_or_method(*py_arguments)
         return self.callback(callback_function, data, level, delay, segmented)
 
@@ -664,6 +742,8 @@ class JSProxyWidget(widgets.DOMWidget):
         assert segmented is None or (type(segmented) is int and segmented > 0), "bad segment " + repr(segmented)
         count = self.counter
         self.counter = count + 1
+        assert not isinstance(callback_function, CommandMakerSuperClass), "can't callback command maker " + type(callback_function)
+        assert not str(data).startswith("Fragile"), "DEBUG::" + repr(data)
         command = CallMaker("callback", count, data, level, segmented)
         #if delay:
         #    callback_function = delay_in_thread(callback_function)
@@ -731,84 +811,90 @@ class JSProxyWidget(widgets.DOMWidget):
         return Loader(LOAD_CSS, css_name, css_text)
 
 
-def validate_commands(commands, top=True):
-    """
-    Validate a command sequence (and convert to list format if needed.)
-    """
-    return [validate_command(c, top) for c in commands]
+    def validate_commands(self, commands, top=True):
+        """
+        Validate a command sequence (and convert to list format if needed.)
+        """
+        return [self.validate_command(c, top) for c in commands]
 
-
-def validate_command(command, top=True):
-    # convert CommandMaker to list format.
-    if isinstance(command, CommandMaker):
-        command = command._cmd()
-    ty = type(command)
-    if ty is list:
-        indicator = command[0]
-        remainder = command[1:]
-        if indicator == "element" or indicator == "window":
-            assert len(remainder) == 0
-        elif indicator == "method":
-            target = remainder[0]
-            name = remainder[1]
-            args = remainder[2:]
-            target = validate_command(target, top=True)
-            assert type(name) is str, "method name must be a string " + repr(name)
-            args = validate_commands(args, top=False)
-            remainder = [target, name] + args
-        elif indicator == "function":
-            target = remainder[0]
-            args = remainder[1:]
-            target = validate_command(target, top=True)
-            args = validate_commands(args, top=False)
-            remainder = [target] + args
-        elif indicator == "id" or indicator == "bytes":
-            assert len(remainder) == 1, "id or bytes takes one argument only " + repr(remainder)
-        elif indicator in LOAD_INDICATORS:
-            assert len(remainder) == 2, "loaders take exactly 2 arguments" + repr(len(remainder))
-        elif indicator == "list":
-            remainder = validate_commands(remainder, top=False)
-        elif indicator == "dict":
-            [d] = remainder
-            d = dict((k, validate_command(d[k], top=False)) for k in d)
-            remainder = [d]
-        elif indicator == "callback":
-            [numerical_identifier, untranslated_data, level, segmented] = remainder
-            assert type(numerical_identifier) is int, \
-                "must be integer " + repr(numerical_identifier)
-            assert type(level) is int, \
-                "must be integer " + repr(level)
-            assert (segmented is None) or (type(segmented) is int and segmented > 0), \
-                "must be None or positive integer " + repr(segmented)
-        elif indicator == "get":
-            [target, name] = remainder
-            target = validate_command(target, top=True)
-            name = validate_command(name, top=False)
-            remainder = [target, name]
-        elif indicator == "set":
-            [target, name, value] = remainder
-            target = validate_command(target, top=True)
-            name = validate_command(name, top=False)
-            value = validate_command(value, top=False)
-            remainder = [target, name, value]
-        elif indicator == "null":
-            [target] = remainder
-            remainder = [validate_command(target, top=False)]
-        else:
-            raise ValueError("bad indicator " + repr(indicator))
-        command = [indicator] + remainder
-    elif top:
-        raise ValueError("top level command must be a list " + repr(command))
-    # Non-lists are untranslated (but should be JSON compatible).
-    return command
+    def validate_command(self, command, top=True):
+        # convert CommandMaker to list format.
+        if isinstance(command, CommandMakerSuperClass):
+            #p("XXXXX CONVERTING COMMAND MAKER", type(command), command)
+            command = command._cmd()
+            #p("XXXXX CONVERTed COMMAND MAKER", type(command), command)
+        elif callable(command):
+            # otherwise convert callables to callbacks, in list format
+            command = self.callable(command)._cmd()
+        assert not isinstance(command, CommandMakerSuperClass), repr((type(command), command))
+        ty = type(command)
+        if ty is list:
+            indicator = command[0]
+            remainder = command[1:]
+            if indicator == "element" or indicator == "window":
+                assert len(remainder) == 0
+            elif indicator == "method":
+                target = remainder[0]
+                name = remainder[1]
+                args = remainder[2:]
+                target = self.validate_command(target, top=True)
+                assert type(name) is str, "method name must be a string " + repr(name)
+                args = self.validate_commands(args, top=False)
+                remainder = [target, name] + args
+                #p("XXXX validated method remainder", remainder)
+            elif indicator == "function":
+                target = remainder[0]
+                args = remainder[1:]
+                target = self.validate_command(target, top=True)
+                args = self.validate_commands(args, top=False)
+                remainder = [target] + args
+            elif indicator == "id" or indicator == "bytes":
+                assert len(remainder) == 1, "id or bytes takes one argument only " + repr(remainder)
+            elif indicator in LOAD_INDICATORS:
+                assert len(remainder) == 2, "loaders take exactly 2 arguments" + repr(len(remainder))
+            elif indicator == "list":
+                remainder = self.validate_commands(remainder, top=False)
+            elif indicator == "dict":
+                [d] = remainder
+                d = dict((k, self.validate_command(d[k], top=False)) for k in d)
+                remainder = [d]
+            elif indicator == "callback":
+                [numerical_identifier, untranslated_data, level, segmented] = remainder
+                assert type(numerical_identifier) is int, \
+                    "must be integer " + repr(numerical_identifier)
+                assert type(level) is int, \
+                    "must be integer " + repr(level)
+                assert (segmented is None) or (type(segmented) is int and segmented > 0), \
+                    "must be None or positive integer " + repr(segmented)
+            elif indicator == "get":
+                [target, name] = remainder
+                target = self.validate_command(target, top=True)
+                name = self.validate_command(name, top=False)
+                remainder = [target, name]
+            elif indicator == "set":
+                [target, name, value] = remainder
+                target = self.validate_command(target, top=True)
+                name = self.validate_command(name, top=False)
+                value = self.validate_command(value, top=False)
+                remainder = [target, name, value]
+            elif indicator == "null":
+                [target] = remainder
+                remainder = [self.validate_command(target, top=False)]
+            else:
+                raise ValueError("bad indicator " + repr(indicator))
+            command = [indicator] + remainder
+        elif top:
+            raise ValueError("top level command must be a list " + repr(command))
+        # Non-lists are untranslated (but should be JSON compatible).
+        return command
 
 def indent_string(s, level, indent="    "):
     lindent = indent * level
     return s.replace("\n", "\n" + lindent)
 
 def to_javascript(thing, level=0, indent=None, comma=","):
-    if isinstance(thing, CommandMaker):
-        return thing.javascript(level)
+    if isinstance(thing, CommandMakerSuperClass):
+        result = thing.javascript(level)
     else:
         ty = type(thing)
         json_value = None
@@ -825,7 +911,9 @@ def to_javascript(thing, level=0, indent=None, comma=","):
             json_value = "Uint8Array(%s)" % inner
         elif json_value is None:
             json_value = json.dumps(thing, indent=indent)
-        return indent_string(json_value, level)
+        result = indent_string(json_value, level)
+    assert type(result) is str, repr((thing, result))
+    return result
 
 
 class ElementWrapper(object):
@@ -840,6 +928,7 @@ class ElementWrapper(object):
     """
 
     def __init__(self, for_widget):
+        assert isinstance(for_widget, JSProxyWidget)
         self.widget = for_widget
         self.widget_element = for_widget.get_element()
 
@@ -857,38 +946,168 @@ class ElementWrapper(object):
         return self.widget(self.widget_element._set(name, value))
 
 class ElementCallWrapper(object):
+    """
+    Wrapper for widget.element.slot_name
+    """
 
     callable_level = 2   # ???
 
     def __init__(self, for_widget, for_element, slot_name):
+        assert isinstance(for_widget, JSProxyWidget)
         self.widget = for_widget
         self.element = for_element
         self.slot_name = slot_name
-    
-    def map_value(self, v):
+
+    def set_context(self):
         widget = self.widget
-        if callable(v):
-            return widget.callable(v, level=self.callable_level)
-        return v
+        self.widget.last_attribute = name
+        setthis = SetMaker(widget.get_element(), FRAGILE_THIS, widget.get_element())
+        setref = SetMaker(
+            widget.get_element(),
+            FRAGILE_JS_REFERENCE,
+            MethodMaker(widget.get_element(), name)
+        )
+        widget.send_commands([setthis, setref])
+    
+    #def map_value(self, v):
+    #    widget = self.widget
+    #    if (not isinstance(v, CommandMakerSuperClass)) and callable(v):
+    #        #p("XXXX mapping", type(v), v)
+    #        return widget.callable(v, level=self.callable_level)
+    #    return v
 
     def __call__(self, *args):
-        mapped_args = map(self.map_value, args)
+        # ("elementcallwrapper.__call__")
+        #mapped_args = map(self.map_value, args)
         widget = self.widget
-        element = self.element
-        slot = element[self.slot_name]
-        widget(slot(*mapped_args))
-        # Allow chaining (may not be consistent with javascript semantics)
-        return element
+        mapped_args = self.widget.wrap_callables(args)
+        #widget(slot(*mapped_args))
+        args = widget.wrap_callables(args)
+        call = CallMaker("method", widget.get_element(), self.slot_name, *args)
+        #p("reset in elementcallwrapper call")
+        widget.last_attribute = None
+        setref = SetMaker(widget.get_element(), FRAGILE_JS_REFERENCE, call)
+        widget.send_command(setref)
+        reference = MethodMaker(widget.get_element(), FRAGILE_JS_REFERENCE)
+        return FragileReference(self.widget, reference, call)
 
     def __getattr__(self, name):
-        for_element = self.element[self.slot_name]
-        return ElementCallWrapper(self.widget, for_element, name)
+        assert isinstance(self.widget, JSProxyWidget)
+        if name=="wrap_callables":
+            raise SystemError(name)
+        widget = self.widget
+        #for_element = self.element[self.slot_name]
+        #get = ElementCallWrapper(self.widget, for_element, name)
+        #get = for_element[name]
+        #return widget.execute_and_return_fragile_reference(get)
+        this_ref = MethodMaker(widget.get_element(), self.slot_name)
+        attr_ref = MethodMaker(this_ref, name)
+        #p ("set in elementcallwrapper getattr", repr(name))
+        self.widget.last_attribute = name
+        set_this = SetMaker(widget.get_element(), FRAGILE_THIS, this_ref)
+        set_ref = SetMaker(widget.get_element(), FRAGILE_JS_REFERENCE, attr_ref)
+        widget.send_commands([set_this, set_ref])
+        reference = MethodMaker(widget.get_element(), FRAGILE_JS_REFERENCE)
+        return FragileReference(self.widget, reference, attr_ref)
 
     # getattr and getitem are the same in Javascript
     __getitem__ = __getattr__
 
+class StaleFragileJavascriptReference(ValueError):
+    "Stale Javascript value reference"
 
-class CommandMaker(object):
+class CommandMakerSuperClass(object):
+    """
+    Superclass for command proxy objects.
+    """
+    pass
+
+class FragileReference(CommandMakerSuperClass):
+
+    def __init__(self, for_widget, referee, cached):
+        assert not isinstance(referee, FragileReference), "bad ref to ref: " + repr(referee)
+        assert isinstance(for_widget, JSProxyWidget)
+        self.widget = for_widget
+        self.referee = referee
+        self.cached = cached
+        self.widget.last_fragile_reference = self
+        #self.widget.last_attribute = None
+
+    def __repr__(self):
+        return "FragileReference(%s)" % id(self.referee)
+
+    def get_protected_content(self):
+        self.error_if_fragile_reference_is_stale()
+        return self.referee
+
+    def error_if_fragile_reference_is_stale(self):
+        #("checking fragile reference\n", repr(self.cached))
+        if self.widget.last_fragile_reference is not self:
+            raise StaleFragileJavascriptReference(
+                    "Value references should only be used in 'chained' expressions."
+                    + "  " + repr(self.cached)
+                )
+
+    def javascript(self, level=0):
+        content = self.get_protected_content()
+        result = content.javascript(level)
+        assert type(result) is str, repr((result, type(content), content))
+        return result
+
+    def _cmd(self):
+        return self.get_protected_content()._cmd()
+
+    def __call__(self, *args):
+        #("fragilereference.__call__", args)
+        self.error_if_fragile_reference_is_stale()
+        widget = self.widget
+        #p ("args", args)
+        args = widget.wrap_callables(args)
+        #return self.referee(*args)
+        # try to call a method of fragile this
+        last_attribute = self.widget.last_attribute
+        if type(last_attribute) is str:
+            action = CallMaker(
+                "method",
+                MethodMaker(widget.get_element(), FRAGILE_THIS),
+                last_attribute,
+                *args
+            )
+        else:
+            # otherwise call the fragile reference as a function
+            action = CallMaker(
+                "function",
+                MethodMaker(widget.get_element(), FRAGILE_JS_REFERENCE),
+                *args
+            )
+        #p("reset in FR __call__")
+        self.widget.last_attribute = None
+        setref = SetMaker(widget.get_element(), FRAGILE_JS_REFERENCE, action)
+        widget.send_command(setref)
+        reference = MethodMaker(widget.get_element(), FRAGILE_JS_REFERENCE)
+        return FragileReference(self.widget, reference, action)
+
+    def __getattr__(self, name):
+        #("fragilereference.__getattr__", name)
+        if name == "_ipython_canary_method_should_not_exist_":
+            #raise AttributeError("what the ...?")
+            return 42 # this will prevent jupyter from poking around in this object (?)
+        self.error_if_fragile_reference_is_stale()
+        #p("set in FR getattr", name)
+        widget = self.widget
+        widget.last_attribute = name
+        this_ref = MethodMaker(self.widget.get_element(), FRAGILE_JS_REFERENCE)
+        attr_ref = MethodMaker(this_ref,  name)
+        set_this = SetMaker(widget.get_element(), FRAGILE_THIS, this_ref)
+        set_ref = SetMaker(widget.get_element(), FRAGILE_JS_REFERENCE, attr_ref)
+        widget.send_commands([set_this, set_ref])
+        reference = MethodMaker(widget.get_element(), FRAGILE_JS_REFERENCE)
+        return FragileReference(widget, reference, attr_ref)
+
+    # getattr and getitem are the same in Javascript
+    __getitem__ = __getattr__
+
+class CommandMaker(CommandMakerSuperClass):
 
     """
     Superclass for command proxy objects.
@@ -954,13 +1173,13 @@ f();
 class SetMaker(CommandMaker):
     """
     Proxy container to set target.name = value.
-    For chaining the result is a reference to the target.
     """
 
     def __init__(self, target, name, value):
         self.target = target
         self.name = name
         self.value = value
+        ##p ("XXXX initialized setmaker: " + self.javascript())
 
     def javascript(self, level=0):
         innerlevel = 2
@@ -1046,14 +1265,14 @@ class CallMaker(CommandMaker):
         # Add newlines in case of long chains.
         if kind == "function":
             function_desc = args[0]
-            function_args = args[1:]
+            function_args = [to_javascript(x) for x in args[1:]]
             function_value = to_javascript(function_desc)
             call_js = "%s\n%s" % (function_value, format_args(function_args))
             return indent_string(call_js, level)
         elif kind == "method":
             target_desc = args[0]
             name = args[1]
-            method_args = args[2:]
+            method_args = [to_javascript(x) for x in args[2:]]
             target_value = to_javascript(target_desc)
             name_value = to_javascript(name)
             method_js = "%s\n[%s]\n%s" % (target_value, name_value, format_args(method_args))
@@ -1080,7 +1299,14 @@ class LiteralMaker(CommandMaker):
     proxy references.
     """
 
-    indicators = {dict: "dict", list: "list", bytearray: "bytes"}
+    indicators = {
+        # things we can translate
+        dict: "dict", list: "list", bytearray: "bytes",
+        # we can't translate non-specific types, modules, etc.
+        type: "don't translate non-specific types, like classes",
+        type(json): "don't translate modules",
+        # xxxx should improve sanity checking on types...
+        }
 
     def __init__(self, thing):
         self.thing = thing
@@ -1095,6 +1321,7 @@ class LiteralMaker(CommandMaker):
         indicator = self.indicators.get(type(thing))
         #return [indicator, thing]
         if indicator:
+            # exact type equality here:
             if ty is list:
                 return [indicator] + quoteLists(thing)
             elif ty is dict:
@@ -1107,6 +1334,8 @@ class LiteralMaker(CommandMaker):
 
 
 def quoteIfNeeded(arg):
+    #if type(arg) is FragileReference:
+    #    arg = arg.get_protected_content()
     if type(arg) in LiteralMaker.indicators:
         return LiteralMaker(arg)
     return arg
@@ -1115,3 +1344,25 @@ def quoteLists(args):
     "Wrap lists or dictionaries in the args in LiteralMakers"
     return [quoteIfNeeded(x) for x in args]
 
+
+class InvalidCommand(Exception):
+    "Invalid command"
+
+
+
+def debug_check_commands(command):
+    "raise an error if the command is not a basic json structure"
+    if command is None:
+        return command
+    ty = type(command)
+    if ty in (int, float, str,  bool):
+        return command
+    if ty in (tuple, list):
+        result = command
+        for x in command:
+            result = debug_check_commands(x)
+        return result
+    if ty is dict:
+        return debug_check_commands(list(command.items()))
+    # otherwise
+    raise InvalidCommand(repr(command))
